/*
 -------------------------------------------------------------------------------
 | Producer - Copyright (C) 2001-2007  Don Burns                               |
 |                                                                             |
 | This library is free software; you can redistribute it and/or modify        |
 | it under the terms of the GNU Lesser General Public License as published    |
 | by the Free Software Foundation; either version 3 of the License, or        |
 | (at your option) any later version.                                         |
 |                                                                             |
 | This library is distributed in the hope that it will be useful, but         |
 | WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  |
 | or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public     |
 | License for more details.                                                   |
 |                                                                             |
 | You should have received a copy of the GNU Lesser General Public License    |
 | along with this software; if not, write to the Free Software Foundation,    |
 | Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.               |
 |                                                                             |
 -------------------------------------------------------------------------------
 */

#include <iostream>

#include "glut.h"

#include <osgDB/ReadFile>

#include <Producer/Camera.h>
#include <Producer/Trackball.h>
#include <ProducerOSG/OsgSceneHandler.h>

#include "GLUTRenderSurface.h"

struct AppInfo 
{
    Producer::ref_ptr<GLUTRenderSurface> glutRenderSurface;
    Producer::ref_ptr<Producer::Camera> camera;
    Producer::ref_ptr<Producer::Trackball> trackball;
    float nx, ny;
    unsigned int mbutton;

    AppInfo(): nx(0.0), ny(0.0), mbutton(0) {}

    void normalizeMouseXY( int mx, int my )
    {
        int wx, wy;
        unsigned int ww, wh;
        glutRenderSurface->getWindowRectangle( wx, wy, ww, wh );

        nx = -1.0 + (2 * (float)mx/(float)ww);
        ny =  1.0 - (2 * (float)my/(float)wh);
    }

}appInfo;

static void frame()
{
    // Update the manipulator
    appInfo.trackball->input(appInfo.nx,appInfo.ny,appInfo.mbutton);
    // Set the camera's view
    appInfo.camera->setViewByMatrix( appInfo.trackball->getMatrix() );
    // Render the frame
    appInfo.camera->frame();
}

void mouseButtonCallback( int button, int state, int x, int y )
{
    if( state == GLUT_DOWN )
        appInfo.mbutton |= (1<<button);
    else
        appInfo.mbutton &= ~(1<<button);

    appInfo.normalizeMouseXY( x, y );
}

void mouseMotionCallback( int x, int y )
{
    appInfo.normalizeMouseXY( x, y );
}

void keyboardCallback( unsigned char key, int, int )
{
    switch( key )
    {
        case ' ':
            appInfo.trackball->reset();
            break;
    }
}

int main( int argc, char **argv )
{
    osg::ref_ptr<osg::Node> scene = osgDB::readNodeFile( "cow.osg" );
    if( !scene.valid() )
    {
        std::cerr << "Can't find the cow!" << std::endl;
        return 1;
    }

    glutInit(&argc, argv);

    appInfo.camera = new Producer::Camera;
    // Set up the RenderSurface
    appInfo.glutRenderSurface = new GLUTRenderSurface( 10, 10, 640, 480);
    appInfo.camera->setRenderSurface( appInfo.glutRenderSurface.get() );

    // Set up the scene handler
    Producer::ref_ptr<Producer::OsgSceneHandler> osgSceneHandler = new Producer::OsgSceneHandler;
    osgSceneHandler->setScene( scene.get() );
    appInfo.camera->setSceneHandler( osgSceneHandler.get() );

    // Initialize the Trackball
    Producer::Vec3 sceneCenter;
    double sceneRadius;
    osgSceneHandler->getSceneBounds( sceneCenter, sceneRadius );

    appInfo.trackball = new Producer::Trackball;
    appInfo.trackball->setOrientation( Producer::Trackball::Z_UP );
    appInfo.trackball->translate(-sceneCenter[0],-sceneCenter[1],-sceneCenter[2]);
    appInfo.trackball->setDistance(sceneRadius*3.0f);
    appInfo.trackball->setReference();

    // Set GLUT callbacks
    glutMouseFunc( mouseButtonCallback );
    glutMotionFunc( mouseMotionCallback );
    glutKeyboardFunc( keyboardCallback );

    // We'll update with an idle callback
    glutIdleFunc( &frame );

    glutMainLoop();
    return 0;
}

